%% Processing cycle
% a leaky integrate and fire neuron in running for ProcessingMinutes simulation time

%% outputs: ExcSynW, InhSynW
% the Weights are saved every simulation second - the low resolution is sufficient for further analysis
ExcSynW = zeros (ExcLearningCount, ProcessingSeconds);
InhSynW = zeros (InhLearningCount, ProcessingSeconds);

%% outputs: PostSynapticRate etc' are saved only when SaveExtraInfo is set by the calling function
% these values are saved every simulation minute - they were left in the code for debugging purposes only
if (SaveExtraInfo)
	PostSynapticRate = zeros (ProcessingMinutes, 1);	% spike count per minute (needs normalization)
	VmMean = zeros (ProcessingMinutes, 1);				% mean membrane potential
	IexcMean = zeros (ProcessingMinutes, 1);			% exc mean current
	IinhMean = zeros (ProcessingMinutes, 1);			% inh mean current
	IleakMean = zeros (ProcessingMinutes, 1);			% leak mean current
	PSTH = zeros (secondTimeBins, ProcessingMinutes);	% spikes over a second time window per minute (needs normalization)
end;
%% outputs: used by the STA analysis code
if (InhStaCount>0)
	InhSTA = zeros (staLen, InhStaCount);
	InhSTASpikesCount = 0;
end;

if (ExcStaCount>0)
	ExcSTA = zeros (staLen, ExcStaCount);
	ExcSTASpikesCount = 0;
end;
  
%% intermediate variables before averaged
Vm = zeros (minuteTimeBins, 1);				% membrane potential
IexcRm = zeros (minuteTimeBins-1, 1);		% exc current * Rm
IinhRm = zeros (minuteTimeBins-1, 1);		% inh current * Rm
IleakRm = zeros (minuteTimeBins-1, 1);		% leak current * Rm

%% Execution loop
SetLearningType; % defines these macros used by the learning mechanism: InhDWantiCausal, ExcDWantiCausal, InhDWcausal, ExcDWcausal

for minute = 1:ProcessingMinutes
	sprintf ('start learning minute %d at time %.1f', minute, toc)

	Vm(1) = cellP.Vrest;
	LastPreInhSpike = zeros (InhSynCount, 1);
	LastPreExcSpike = zeros (ExcSynCount, 1);
	PostSynapticSpikesTimes = NegligibleHistory*ones(PostSynapticSpikes4calc, 1);
	NextPostSynapticSpikeP = 1;

	CurLambda = LearningLambdaPerMinute(minute);
	GenSynapticInput;

	for t = 1:minuteTimeBins-1
		tSecOffset = mod (t, secondTimeBins);
		InhDWpre = zeros (InhLearningCount,1);
		InhDWpost = zeros (InhLearningCount,1);
		ExcDWpre = zeros (ExcLearningCount,1);
		ExcDWpost = zeros (ExcLearningCount,1);
		
		% Inh pre syn spike learning
		for i = 1:InhLearningCount
			if (InhSyn(i).SpikesTimes(LastPreInhSpike(i)+1) == t)
				LastPreInhSpike(i) = LastPreInhSpike(i)+1;
				InhDWpre(i) = InhDWantiCausal (CurLambda, InhSynWeights(i), PostSynapticSpikesTimes-t); % anti-causal times are negative
			end;
		end;
		% Exc pre syn spike learning
		for i = 1:ExcLearningCount
			if (ExcSyn(i).SpikesTimes(LastPreExcSpike(i)+1) == t)
				LastPreExcSpike(i) = LastPreExcSpike(i)+1;
				ExcDWpre(i) = ExcDWantiCausal (CurLambda, ExcSynWeights(i), PostSynapticSpikesTimes-t); % anti-causal times are negative
			end;
		end;

		IexcRm(t) = (cellP.Eexc-Vm(t))*ExcSynGn(t,:)*ExcSynWeights;
		IinhRm(t) = (cellP.Einh-Vm(t))*InhSynGn(t,:)*InhSynWeights;
		IleakRm(t) = (cellP.Vrest-Vm(t));
		Vm(t+1) = Vm(t)+dtOverTauMembrane*(IexcRm(t)+IinhRm(t)+IleakRm(t));

		if (Vm(t+1) >= cellP.Eth)
			% a post synaptic spike has occured
			Vm(t+1) = cellP.Vrest;
			PostSynapticSpikesTimes(NextPostSynapticSpikeP) = t+0.5; % the average spike time is the middle of the time bin 
			NextPostSynapticSpikeP = 1 + mod (NextPostSynapticSpikeP, PostSynapticSpikes4calc);

			if (SaveExtraInfo)
				PostSynapticRate(minute)=PostSynapticRate(minute)+1;
				if (tSecOffset == 0)
					PSTH(end, minute) = PSTH(end, minute) + 1;
				else
					PSTH(tSecOffset, minute) = PSTH(tSecOffset, minute) + 1;
				end;
			end;

			% Inh post syn spike learning
			for i = 1:InhLearningCount
				if (LastPreInhSpike(i)>0)
					FirstSpike4calc = max (LastPreInhSpike(i)-PreSynapticSpikes4calc, 1);
					InhDWpost(i) = InhDWcausal (CurLambda, InhSynWeights(i),...
						t+0.5-InhSyn(i).SpikesTimes(FirstSpike4calc:LastPreInhSpike(i))); % causal times are positive
				end;
			end;
			% Exc post syn spike learning
			for i = 1:ExcLearningCount
				if (LastPreExcSpike(i)>0)
					FirstSpike4calc = max (LastPreExcSpike(i)-PreSynapticSpikes4calc, 1);
					ExcDWpost(i) = ExcDWcausal (CurLambda, ExcSynWeights(i),...
						t+0.5-ExcSyn(i).SpikesTimes(FirstSpike4calc:LastPreExcSpike(i))); % causal times are positive
				end;
			end;
			
			% STA summing
			if (InhStaCount>0 && t>staLen+1)
				InhSTA = InhSTA+InhSpikeTrain(t-staLen+1:t,1:InhStaCount);
				InhSTASpikesCount = InhSTASpikesCount+1;
			end;
			if (ExcStaCount>0 && t>staLen+1)
				ExcSTA = ExcSTA+ExcSpikeTrain(t-staLen+1:t,1:ExcStaCount);
				ExcSTASpikesCount = ExcSTASpikesCount+1;
			end;
		end;
		ExcSynWeights(1:ExcLearningCount) = min (max (ExcSynWeights(1:ExcLearningCount)+ExcDWpre+ExcDWpost, 0), 1);
		InhSynWeights(1:InhLearningCount) = min (max (InhSynWeights(1:InhLearningCount)+InhDWpre+InhDWpost, 0), 1);
		if (tSecOffset == 0)
			ExcSynW (:, (minute-1)*60+t*dt) = ExcSynWeights(1:ExcLearningCount);
			InhSynW (:, (minute-1)*60+t*dt) = InhSynWeights(1:InhLearningCount);
		end;
	end;
	ExcSynW (:, minute*60) = ExcSynWeights(1:ExcLearningCount);
	InhSynW (:, minute*60) = InhSynWeights(1:InhLearningCount);

	if (SaveExtraInfo)
		VmMean(minute) = mean (Vm);
		IexcMean(minute) = mean (IexcRm)/cellP.Rm;
		IinhMean(minute) = mean (IinhRm)/cellP.Rm;
		IleakMean(minute) = mean (IleakRm)/cellP.Rm;
	end;
end;
